(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uordered_field.ml,v 1.18 2017/11/07 20:57:39 fclement Exp $ *)

(* Shortcuts for the Ulib library. *)

module Nat = Unatural;;
module P = Upervasives;;

(* Ordered field operations. *)

module Make =
  functor (S : Uordered_field_spec.STRUCTURE) ->
struct
  include S;;

  let _2_ = _1_ +. _1_;;
  let _3_ = _2_ +. _1_;;
  let _4_ = _3_ +. _1_;;
  let _5_ = _4_ +. _1_;;
  let _6_ = _5_ +. _1_;;
  let _7_ = _6_ +. _1_;;
  let _8_ = _7_ +. _1_;;
  let _9_ = _8_ +. _1_;;

  let minus_1_ = ~-. _1_
  and minus_2_ = ~-. _2_
  and minus_3_ = ~-. _3_
  and minus_4_ = ~-. _4_
  and minus_5_ = ~-. _5_
  and minus_6_ = ~-. _6_
  and minus_7_ = ~-. _7_
  and minus_8_ = ~-. _8_
  and minus_9_ = ~-. _9_
  ;;

  let inv_2_ = ~/. _2_
  and inv_3_ = ~/. _3_
  and inv_4_ = ~/. _4_
  and inv_5_ = ~/. _5_
  and inv_6_ = ~/. _6_
  and inv_7_ = ~/. _7_
  and inv_8_ = ~/. _8_
  and inv_9_ = ~/. _9_
  ;;

  let ( -. ) x y = x +. (~-. y);;
  let ( /. ) x y = x *. (~/. y);;
  let axpy a x y = a *. x +. y;;
  let mean x y = (x +. y) /. _2_;;
  let square x = x *. x;;

  let of_nat n = of_int (Nat.to_int n);;
  let to_nat x = Nat.of_int (to_int x);;
end
;;


module Float_structure =
struct
  type t = float;;

  let printer = Og_printer.printer_Float;;
  let print ppf = printer Og_precedence.Ck_toplevel ppf;;
  let printf oc = Printf.fprintf oc "%F";;

  let _0_ = 0.
  and _1_ = 1.
  ;;

  let abs = abs_float;;

  let is_zero eps x = abs x <= eps;;

  let ( +. ) = Pervasives.( +. );;
  let ( ~-. ) = Pervasives.( ~-. );;
  let ( *. ) = Pervasives.( *. );;
  let ( ~/. ) x = Pervasives.( /. ) _1_ x;;

  external of_float : float -> t = "%identity";;
  external to_float : t -> float = "%identity";;

  let of_int = Pervasives.float_of_int;;
  let to_int = P.round;;

  let of_bit_string = P.float_of_bit_string;;
  let to_bit_string = P.bit_string_of_float;;

  let compare = Pervasives.compare;;
end
;;

module Float = Make (Float_structure);;
