(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uindex_numbering.ml,v 1.5 2017/11/07 20:57:40 fclement Exp $*)

(* Shortcuts for the Ulib library. *)

module A = Uarray;;

(* Shortcuts for the Mapping library. *)

module Num = Unumbering;;
module Iidx = Uindex_index;;

(* Index numbering bijection operations. *)

type 'a key = 'a Iidx.element;;
type 'a value = 'a Iidx.index;;

type 'a t = 'a key Num.t;;

external of_t : 'a t -> 'a key Num.t = "%identity";;

(* Construction function. *)

let make a =
  A.check_nonempty "Uindex_numbering.make" a;
  A.check_strictly_increasing "Uindex_numbering.make" a;
  let len = A.length a in
  let h_dir k = Iidx.of_nat (A.sorted_array_find a k) in
  let h_inv v = A.get a (Iidx.to_nat v) in
  { Num.length = len; h_dir = h_dir; h_inv = h_inv; }
;;

(* Access functions. *)

let length = Num.length;;

let mem ih k =
  try ignore (ih.Num.h_dir k); true with
  | Not_found -> false
;;

let value_of_key = Num.value_of_key;;

let key_of_value = Num.key_of_value;;

let equal = Num.equal;;

(* Iterators. *)

let iter f = Num.iter f;;

let sub_iter f = Num.sub_iter f;;

let fold_left f = Num.fold_left f;;
