(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uindex_numbering.mli,v 1.2 2017/05/31 17:27:38 fclement Exp $*)

(** {3 Index numbering bijections}

 Specialized numbering bijections for numbering indices, i.e. natural numbers.
 In this case, a costless bijective numbering function can be defined.
 Type [Uindex_numbering.t] is an efficient alternative to
 [(Unatural.t, Unatural.t) Hashtbl.t]. *)

type 'a key = 'a Uindex_index.element;;
type 'a value = 'a Uindex_index.index;;

type 'a t = private 'a key Unumbering.t;;

external of_t : 'a t -> 'a key Unumbering.t = "%identity";;

(** {6 Construction function} *)

val make : 'a key array -> 'a t
(** The array argument must be nonempty, and its elements must be strictly
 increasing. *)
;;

(** {6 Access functions} *)

val length : 'a t -> Upervasives.length;;

val mem : 'a t -> 'a key -> bool;;

val value_of_key : 'a t -> 'a key -> 'a value;;

val key_of_value : 'a t -> 'a value -> 'a key;;

val equal : 'a t -> 'a t -> bool
(** Structural equality of index hash tables. *)
;;

(** {6 Iterators} *)

val iter : ('a key -> 'a value -> unit) -> 'a t -> unit;;

val sub_iter :
  ('a key -> 'a value -> unit) ->
  'a t -> 'a value -> Upervasives.length -> unit
;;

val fold_left :
  ('accu -> 'a key -> 'a value -> 'accu) -> 'accu -> 'a t -> 'accu
;;
