(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: umatrix_universe.mli,v 1.12 2017/10/27 13:30:02 fclement Exp $*)

(** {3 Numbering matrix slots} *)

type size = Umatrix_numbering.size;;
type element = Umatrix_numbering.key;; (* = Unatural.t * Unatural.t *)
type index = Umatrix_numbering.value;; (* = element Uindex.t *)

type t = private
  {
   tables : Umatrix_numbering.t;
   cardinal : Upervasives.length;
  }
(** Any set we consider in this library is a subset of some universe.
 More precisely, a universe is the support of a finite family of finite sets.
 It is a set that contains all the elements that belong to all the sets of
 the family. The union of all sets of a family of sets is the smallest
 universe for this family.
*)
;;

(** {6 Construction functions} *)

val make : size -> t;;

val of_mask : Umatrix_mask.t -> t;;
val make_masked : Umatrix_mask.selection_matrix -> t;;

(** {6 Access functions} *)

val cardinal : t -> Upervasives.length;;
val size : t -> size;;

val index : t -> element -> index;;

val element : t -> index -> element;;
val sub_elements : t -> index array -> element array;;
