(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: unumbering_print.ml,v 1.7 2017/11/07 20:57:40 fclement Exp $*)

(* Manually edited printers for numbering bijections. *)

let cktl = Og_precedence.Ck_toplevel;;

open Unumbering;;

let pprint_value = Uindex_print.printer_t;;

let pprint_entry pp_x pp_fx f _ctx ppf x _fx =
  Format.fprintf ppf "@ @[| %a ->@ %a@]"
    (pp_x cktl) x
    (pp_fx cktl) (f x);
;;

let pprint_entry_dir pp_key pp_key_value =
  pprint_entry pp_key pp_key_value
;;

let pprint_h_dir pp_key pp_key_value _ctx ppf t =
  Format.fprintf ppf "%a"
    (fun ppf -> iter (pprint_entry_dir pp_key pp_key_value t.h_dir cktl ppf))
    t
;;

let pprint_entry_inv pp_key pp_key_value h_inv ctx ppf k v =
  pprint_entry pp_key_value pp_key h_inv ctx ppf v k
;;

let pprint_h_inv pp_key pp_key_value _ctx ppf t =
  Format.fprintf ppf "%a"
    (fun ppf -> iter (pprint_entry_inv pp_key pp_key_value t.h_inv cktl ppf))
    t
;;

(* It would have been very satisfatory to automatically generate the following
 printer, but pprint_h_dir and pprint_h_inv above need the iterator
 Unumbering.iter which can only be defined after the definition of the type
 Unumbering.t.
 Hence, we cannot define ``small'' modules Bijections and Bijections_print that
 only provides types 'key h_dir, 'key h_inv and the associated printers
 above. *)

let pprint_t pp_key ctx ppf ({ length = u_l; _; } as t) =
  let pp_key_value = pprint_value pp_key in
  if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
    then Format.fprintf ppf "%, {@ " else
  Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";

  Format.fprintf ppf "%,@[<hv 2>length =";
  Upervasives_print.printer_length Og_precedence.Ck_record_field ppf u_l;
  Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";

  Format.fprintf ppf "%,@[<hv 2>h_dir : key -> value =";
  pprint_h_dir pp_key pp_key_value Og_precedence.Ck_record_field ppf t;
  Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";

  Format.fprintf ppf "%,@[<hv 2>h_inv : value -> key =";
  pprint_h_inv pp_key pp_key_value Og_precedence.Ck_record_field ppf t;
  Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";

  if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
    then Format.fprintf ppf "%,@;<1 -2>}" else
  Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_value = pprint_value
and printer_t = pprint_t
;;

let print_value pp_a = printer_value pp_a cktl
and print_t pp_key = printer_t pp_key cktl
;;
