(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: black_box.ml,v 1.73 2017/11/07 20:57:40 fclement Exp $ *)

module Config = Configuration;;
module W = Wrapper;;

let piob_ref = ref None;;

let pf = Pio.foreman_make_printing_facilities ();;

module Make =
  functor (Scal : Scalar_spec.SCALAR) ->
  functor (Comps : Components_spec.COMPONENTS) ->
struct
  module T = Translate.Make (Scal) (Comps);;
  module Stt = T.Stt;;

  type scalar = Scal.t;;
  type 'a components = 'a Comps.t;;

  type gradients = Stt.gradients;;
  type partitions = Stt.partitions;;
  type state = Stt.state;;
  type selected_refinement = Stt.selected_refinement;;

  let wrap tw =
    let cmd = Config.get_external_program_command () in
    Pio.foreman_wrap_task cmd piob_ref tw
  ;;

  (* Init. *)

  let initialize x = wrap (W.make_init_wrapper pf "init") x;;

  let init_state () =
    let dim = T.inj_init () in
    let init_out = initialize dim in
    let res = T.proj_coarse_init init_out in
    res
  ;;

  let compute_gradient x = wrap (W.make_grad_wrapper pf "grad") x;;

  let init_constants ({ Stt.cost = j0; _; } as st0) =
    let (t_inj_fine, _) = T.make_fine st0 in
    let pm0 = t_inj_fine () in
    let gm0 = compute_gradient pm0 in
    T.proj_constants j0 gm0
  ;;

  (* Grad. *)

  let grad st =
    let (t_inj_fine, t_proj_fine) = T.make_fine st in
    let pm = t_inj_fine () in
    let gm = compute_gradient pm in
    t_proj_fine gm
  ;;

  (* Optim. *)

  let optimize x = wrap (W.make_optim_wrapper pf "optim") x;;

  let optim st_init pps_init =
    let (t_inj_coarse, t_proj_coarse) = T.make_coarse st_init pps_init in
    let (ppm, maa_init) = t_inj_coarse () in
    let optim_out_opt = optimize (ppm, maa_init) in
    t_proj_coarse optim_out_opt
  ;;

  (* Final. *)

  let finalize x = wrap (W.make_final_wrapper pf "final") x;;

  let final st =
    let (t_inj_fine, _) = T.make_fine st in
    let pm = t_inj_fine () in
    finalize pm
  ;;
end
;;
