(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: black_box_spec.mli,v 1.31 2017/11/07 20:57:40 fclement Exp $ *)

(** {3 Interface to the external part of State maker functor} *)

open Refinement_types;;

type constants = Constants_types.t;;

module type MAKER =
  functor (Scal : Scalar_spec.SCALAR) ->
  functor (Comps : Components_spec.COMPONENTS) ->
sig
  module Stt : module type of (State_types.Make (Scal) (Comps));;

  type scalar = Scal.t;;
  type 'a components = 'a Comps.t;;

  type gradients = Stt.gradients;;
  type partitions = Stt.partitions;;
  type state = Stt.state;;
  type selected_refinement = Stt.selected_refinement;;

  val init_state : unit -> state;;
  val init_constants : state -> constants;;
  val final : state -> unit;;

  val grad : state -> gradients * gradients_norm;;
  val optim : state -> partitions -> selected_refinement;;
end
;;
