(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: configuration.mli,v 1.26 2017/11/13 12:40:10 fclement Exp $ *)

open Refinement_types;;
open Display_types;;

type t = Configuration_types.t;;

(** {6 Uhistory.Make functor API} *)

val printer : t Og_types.printer;;
val print : t Og_types.top_printer;;

val cold_set : unit -> unit;;
val warm_set : t -> unit;;
val get : unit -> t;;

val phase_dimension : Ucommand_option.phase_name;;

(** {6 Algorithm configuration} *)

val get_external_program_command : unit -> external_program_command;;
val get_dimension : unit -> dimension;;
val get_dimension_management : unit -> dimension_management;;
val get_refinement_strategy : unit -> refinement_strategy;;
val get_candidate_refinements_selection_criterion :
  unit -> candidate_refinements_selection_criterion
;;
val get_stopping_criterion_config : unit -> stopping_criterion_config;;

(** {6 Program configuration} *)

val get_formatter : unit -> Format.formatter;;
val get_report_allocated_memory : unit -> bool;;
val get_incremental_mode : unit -> bool;;
val get_graphics_mode : unit -> graphics_mode;;
val get_graphics_window_min_size : unit -> size;;
val get_graphics_window_max_size : unit -> size;;
val get_graphics_window_bg_color : unit -> color;;
val get_graphics_parameter_color_mapping : unit -> Colormapping.t;;
val get_graphics_gradient_color_mapping : unit -> Colormapping.t;;

(** {6 Monitoring} *)

val get_warn_error : unit -> bool;;

val report : Upervasives.message -> unit;;
val debug : Upervasives.function_name -> Upervasives.message -> unit;;
val warning : Upervasives.function_name -> Upervasives.message -> unit;;
val error : Upervasives.function_name -> Upervasives.message -> unit;;

val report_action : Upervasives.gen_message -> unit;;
val debug_action :
  Upervasives.function_name -> Upervasives.gen_message -> unit
;;
val warning_action :
  Upervasives.function_name -> Upervasives.gen_message -> unit
;;
val error_action :
  Upervasives.function_name -> Upervasives.gen_message -> unit
;;

val print_ident_value : string -> ('a, 'a) Og_types.top_printer_lifter;;

val print_report : ('a, 'a) Og_types.top_printer_lifter;;
val print_debug :
  Upervasives.function_name -> string -> ('a, 'a) Og_types.top_printer_lifter
;;
val print_debug_no_arg :
  Upervasives.function_name ->
  (Format.formatter -> unit) -> Format.formatter -> unit
;;
