(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* Beginning of OCamlGen, 4.2+dev0 generated code. *)

let printer_component_number ctx ppf = function
  | u_t -> Unatural_print.printer_t ctx ppf u_t
;;

let printer_point ctx ppf = function
  | (u_t, u_t0) ->
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
      else
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@[<hv 2>(@,";
    Format.fprintf ppf "%,@[<hv 0>";
    Unatural_print.printer_t Og_precedence.Ck_tuple_item ppf u_t;
    Format.fprintf ppf "%,,@ ";
    Unatural_print.printer_t Og_precedence.Ck_tuple_item ppf u_t0;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
      else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@;<0 -2>)@]"
;;

let printer_parameter pp_a ctx ppf = function
  | a_u_m ->
    Umatrix_structural_mapping_print.printer_mapping pp_a ctx ppf a_u_m

and printer_sub_parameter pp_a ctx ppf = function
  | a_u_s ->
    Umatrix_structural_mapping_print.printer_sub_mapping pp_a ctx ppf a_u_s
;;

let printer_gradient pp_a ctx ppf = function
  | a_p -> printer_parameter pp_a ctx ppf a_p
;;

let rec printer_cost ctx ppf = function
  | f -> Og_printer.printer_Float ctx ppf f

and printer_first_order_indicator pp_a ctx ppf = function
  | a -> pp_a ctx ppf a

and printer_exact_indicator ctx ppf = function | c -> printer_cost ctx ppf c

and printer_gradients_norm ctx ppf = function
  | f -> Og_printer.printer_Float ctx ppf f

and printer_initial_gradients_norm ctx ppf = function
  | g -> printer_gradients_norm ctx ppf g
;;

let printer_candidate_refinement pp_a ctx ppf = function
  | {
      Refinement_types.refinement = u_d;
      Refinement_types.first_order_indicator = a_f;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,@[<hv 2>refinement =";
    Umatrix_nonempty_set_print.printer_dichotomy
      Og_precedence.Ck_record_field
      ppf
      u_d;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>first_order_indicator =";
    printer_first_order_indicator pp_a Og_precedence.Ck_record_field ppf a_f;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_zone pp_a ctx ppf = function
  | a_a_c_l_u_b ->
    Umatrix_structural_mapping_print.printer_block
      pp_a
      (Og_printer.printer_List (printer_candidate_refinement pp_a))
      ctx
      ppf
      a_a_c_l_u_b
;;

let printer_zone_number ctx ppf = function
  | u_b -> Umatrix_structural_mapping_print.printer_block_number ctx ppf u_b
;;

let printer_bounds ctx ppf = function
  | { Refinement_types.ll = p; Refinement_types.ur = p0; } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,@[<hv 2>ll =";
    printer_point Og_precedence.Ck_record_field ppf p;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>ur =";
    printer_point Og_precedence.Ck_record_field ppf p0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_zonation pp_a ctx ppf = function
  | a_a_c_l_a_g_u_t ->
    Umatrix_structural_mapping_print.printer_t
      pp_a
      (Og_printer.printer_List (printer_candidate_refinement pp_a))
      (printer_gradient pp_a)
      ctx
      ppf
      a_a_c_l_a_g_u_t
;;

let printer_forced_zone pp_a ctx ppf = function
  | a_s_a_c_l_u_a ->
    Udynamic_graph_print.printer_arrow
      (printer_sub_parameter pp_a)
      (Og_printer.printer_List (printer_candidate_refinement pp_a))
      ctx
      ppf
      a_s_a_c_l_u_a
;;

let printer_forced_zonation pp_a ctx ppf = function
  | a_f_a ->
    Og_printer.printer_Array (printer_forced_zone pp_a) ctx ppf a_f_a
;;

let printer_partition pp_a ctx ppf = function
  | a_s_a ->
    Og_printer.printer_Array (printer_sub_parameter pp_a) ctx ppf a_s_a
;;

let printer_command_line ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s
;;

let printer_external_program_command ctx ppf = function
  | c -> printer_command_line ctx ppf c
;;

let printer_dimension ctx ppf = function
  | u_t -> Unatural_print.printer_t ctx ppf u_t
;;

let printer_dimension_management _ctx ppf = function
  | Refinement_types.Vector ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Vector"
  | Refinement_types.Best_component_only ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Best_component_only"
;;

let printer_number_of_cuts ctx ppf = function
  | u_t -> Unatural_print.printer_t ctx ppf u_t
;;

let printer_basic_refinement_family _ctx ppf = function
  | Refinement_types.Horizontal_cut ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Horizontal_cut"
  | Refinement_types.Vertical_cut ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Vertical_cut"
;;

let printer_refinement_strategy ctx ppf = function
  | Refinement_types.Best_in_a_family (n, b_u_l) ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Best_in_a_family";
    Format.fprintf ppf "%, (@,"; Format.fprintf ppf "%,@[<hv 0>";
    printer_number_of_cuts Og_precedence.Ck_tuple_item ppf n;
    Format.fprintf ppf "%,,@ ";
    Ulist1_print.printer_list1 printer_basic_refinement_family
      Og_precedence.Ck_tuple_item
      ppf
      b_u_l;
    Format.fprintf ppf "%,@]"; Format.fprintf ppf "%,@;<0 -2>)";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Refinement_types.Overall_best ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Overall_best"
;;

let printer_candidate_refinements_selection_criterion ctx ppf = function
  | (f, u_t) ->
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
      else
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@[<hv 2>(@,";
    Format.fprintf ppf "%,@[<hv 0>";
    Og_printer.printer_Float Og_precedence.Ck_tuple_item ppf f;
    Format.fprintf ppf "%,,@ ";
    Unatural_print.printer_t Og_precedence.Ck_tuple_item ppf u_t;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
      else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@;<0 -2>)@]"
;;

let printer_stopping_criterion_config ctx ppf = function
  | {
      Refinement_types.min_relative_gradients_norm = f;
      Refinement_types.min_relative_best_scalar_indicator = f0;
      Refinement_types.min_relative_scalar_cost_decrease = f1;
      Refinement_types.max_iteration = u_s;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,@[<hv 2>min_relative_gradients_norm =";
    Og_printer.printer_Float Og_precedence.Ck_record_field ppf f;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>min_relative_best_scalar_indicator =";
    Og_printer.printer_Float Og_precedence.Ck_record_field ppf f0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>min_relative_scalar_cost_decrease =";
    Og_printer.printer_Float Og_precedence.Ck_record_field ppf f1;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>max_iteration =";
    Uhistory_types_print.printer_state_number
      Og_precedence.Ck_record_field
      ppf
      u_s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_graphics_mode _ctx ppf = function
  | Refinement_types.No_graphics ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,No_graphics"
  | Refinement_types.Parameter_graphics ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Parameter_graphics"
  | Refinement_types.Full_graphics ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Full_graphics"
;;

let printer_output_status ctx ppf = function
  | Refinement_types.Max_iteration u_s ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Max_iteration";
    Uhistory_types_print.printer_state_number
      Og_precedence.Ck_argument
      ppf
      u_s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Refinement_types.Min_relative_best_scalar_indicator f ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Min_relative_best_scalar_indicator";
    Og_printer.printer_Float Og_precedence.Ck_argument ppf f;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Refinement_types.Min_relative_scalar_cost_decrease f ->
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>Min_relative_scalar_cost_decrease";
    Og_printer.printer_Float Og_precedence.Ck_argument ppf f;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
  | Refinement_types.No_more_refinements ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,No_more_refinements"
;;

(* Defining top printers. *)

let print_component_number ppf =
  printer_component_number Og_precedence.Ck_toplevel ppf
;;

let print_point ppf = printer_point Og_precedence.Ck_toplevel ppf
;;

let print_parameter pp_a ppf =
  printer_parameter pp_a Og_precedence.Ck_toplevel ppf

and print_sub_parameter pp_a ppf =
  printer_sub_parameter pp_a Og_precedence.Ck_toplevel ppf
;;

let print_gradient pp_a ppf =
  printer_gradient pp_a Og_precedence.Ck_toplevel ppf
;;

let print_cost ppf = printer_cost Og_precedence.Ck_toplevel ppf

and print_first_order_indicator pp_a ppf =
  printer_first_order_indicator pp_a Og_precedence.Ck_toplevel ppf

and print_exact_indicator ppf =
  printer_exact_indicator Og_precedence.Ck_toplevel ppf

and print_gradients_norm ppf =
  printer_gradients_norm Og_precedence.Ck_toplevel ppf

and print_initial_gradients_norm ppf =
  printer_initial_gradients_norm Og_precedence.Ck_toplevel ppf
;;

let print_candidate_refinement pp_a ppf =
  printer_candidate_refinement pp_a Og_precedence.Ck_toplevel ppf
;;

let print_zone pp_a ppf = printer_zone pp_a Og_precedence.Ck_toplevel ppf
;;

let print_zone_number ppf = printer_zone_number Og_precedence.Ck_toplevel ppf
;;

let print_bounds ppf = printer_bounds Og_precedence.Ck_toplevel ppf
;;

let print_zonation pp_a ppf =
  printer_zonation pp_a Og_precedence.Ck_toplevel ppf
;;

let print_forced_zone pp_a ppf =
  printer_forced_zone pp_a Og_precedence.Ck_toplevel ppf
;;

let print_forced_zonation pp_a ppf =
  printer_forced_zonation pp_a Og_precedence.Ck_toplevel ppf
;;

let print_partition pp_a ppf =
  printer_partition pp_a Og_precedence.Ck_toplevel ppf
;;

let print_command_line ppf =
  printer_command_line Og_precedence.Ck_toplevel ppf
;;

let print_external_program_command ppf =
  printer_external_program_command Og_precedence.Ck_toplevel ppf
;;

let print_dimension ppf = printer_dimension Og_precedence.Ck_toplevel ppf
;;

let print_dimension_management ppf =
  printer_dimension_management Og_precedence.Ck_toplevel ppf
;;

let print_number_of_cuts ppf =
  printer_number_of_cuts Og_precedence.Ck_toplevel ppf
;;

let print_basic_refinement_family ppf =
  printer_basic_refinement_family Og_precedence.Ck_toplevel ppf
;;

let print_refinement_strategy ppf =
  printer_refinement_strategy Og_precedence.Ck_toplevel ppf
;;

let print_candidate_refinements_selection_criterion ppf =
  printer_candidate_refinements_selection_criterion
    Og_precedence.Ck_toplevel
    ppf
;;

let print_stopping_criterion_config ppf =
  printer_stopping_criterion_config Og_precedence.Ck_toplevel ppf
;;

let print_graphics_mode ppf =
  printer_graphics_mode Og_precedence.Ck_toplevel ppf
;;

let print_output_status ppf =
  printer_output_status Og_precedence.Ck_toplevel ppf
;;

(* End of OCamlGen, 4.2+dev0 generated code. *)
