CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                              Ref-indic                              C
C                                                                     C
C                        Refinement indicators                        C
C                                                                     C
C                       EPI Serena, INRIA Paris                       C
C                                                                     C
C  Copyright (c) 2013-2017 INRIA.                                     C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C     $Id: model.fpp,v 1.9 2017/11/07 23:47:13 fclement Exp $

#include "model_constants.h"

      subroutine init
     +     (channel_number, width, height, area, selected_size,
     +     ier, len, msg)
C     Initialize the model.

      implicit none

C     Output arguments.
      character msg*(*)
      integer channel_number, width, height, area, selected_size
      integer ier, len

      call parse_config_file(ier, len, msg)
      if (ier .ne. 0) return

      call load_pnm_image
     +     (channel_number, width, height, area, selected_size,
     +     ier, len, msg)
      if (ier .ne. 0) return

      return
      end


      subroutine parse_config_file(ier, len, msg)
C     Parse the configuration file.

      implicit none

      include 'model_definition.fh'

C     Output arguments.
      character msg*(*)
      integer ier, len

C     Local variables.
      character options_fname*14
      logical use_mask_option
      integer of_len

      of_len = 14
      options_fname = 'model_config'

      open(unit=IOUNIT, file=options_fname, status='old',
     +     access='sequential', form='formatted', iostat=ier)
      if (ier .ne. 0) then
         len = 22 + of_len + 1
         msg(1:len) =
     +        'open failure on file "'//options_fname(1:of_len)//'"'
         return
      endif

      read(IOUNIT, *) fni_len
      if (fni_len .gt. MAX_FILE_NAME_LENGTH) then
         len = 27
         msg(1:len) = 'input file name is too long'
         return
      endif

      read(IOUNIT, *) fname_in(1:fni_len)
      write(OUTPUT, *)
     +     'input file name = ''', fname_in(1:fni_len), ''''

C     Default is no mask.
      call unset_use_mask
      read(IOUNIT, *) use_mask_option
      if (use_mask_option) call set_use_mask

      read(IOUNIT, *) fno_len
      if (fno_len .gt. MAX_FILE_NAME_LENGTH) then
         len = 28
         msg(1:len) = 'output file name is too long'
         return
      endif

      read(IOUNIT, *) fname_out(1:fno_len)
      write(OUTPUT, *)
     +     'output file name = ''', fname_out(1:fno_len), ''''

      close(IOUNIT)

      return
      end


      subroutine load_pnm_image
     +     (channel_number, width, height, area, selected_size,
     +     ier, len, msg)
C     Load PNM image (only accepts a subset of PNM images).

      implicit none

      include 'model_definition.fh'

C     Output arguments.
      character msg*(*)
      integer channel_number, width, height, area, selected_size
      integer ier, len

C     Local variables.
      character magic*2

      open(unit=IOUNIT, file=fname_in(1:fni_len),
     +     status='old', access='sequential', form='formatted',
     +     iostat=ier)
      if (ier .ne. 0) then
         len = 22 + fni_len + 1
         msg(1:len) =
     +        'open failure on file "'//fname_in(1:fni_len)//'"'
         return
      endif

      read(IOUNIT, *) magic

      if ((magic .eq. 'P1') .or. (magic .eq. 'P4') .or.
     +     (magic .eq. 'P2') .or. (magic .eq. 'P5')) then
         dim_d = 1
      else if ((magic .eq. 'P3') .or. (magic .eq. 'P6')) then
         dim_d = 3
      else if (magic .eq. 'P7') then
         dim_d = 4
      else
         ier = 1
         len = 22 + 2 + 1
         msg(1:len) = 'unbound magic number "'//magic//'"'
         return
      endif

      if (dim_d .gt. MAX_CHANNEL_NUMBER) then
         call too_large(5, 'dim_d', dim_d, ier, len, msg)
         return
      endif
      channel_number = dim_d

      write(OUTPUT, 1001) channel_number

 1001 format('channel_number = ', i2)

      if ((magic .eq. 'P1') .or.
     +     (magic .eq. 'P2') .or.
     +     (magic .eq. 'P3')) then
         call load_ascii_pnm
     +        (width, height, area, selected_size, ier, len, msg)
         if (ier .ne. 0) return
      else if ((magic .eq. 'P4') .or.
     +     (magic .eq. 'P5') .or.
     +     (magic .eq. 'P6')) then
         ier = 1
         len = 32
         msg(1:len) = 'load raw PNM not yet implemented'
         return
      else if (magic .eq. 'P7') then
         ier = 1
         len = 28
         msg(1:len) = 'load PAM not yet implemented'
         return
      endif

      close(IOUNIT)

      return
      end


      subroutine load_ascii_pnm
     +     (width, height, area, selected_size, ier, len, msg)
C     Load ASCII PNM image (only accepts a subset of ASCII PNM images).

      implicit none

      include 'model_definition.fh'

C     Output arguments.
      character msg*(*)
      integer width, height, area, selected_size, ier, len

C     Local variables.
      character comment*1
      integer ix, iy, i, j, k
      real*8 maxval, line(3 * MAX_WIDTH)

C     External function.
      logical get_use_mask

      read(IOUNIT, *) comment
      if (comment .ne. '#') then
         ier = 1
         len = 16
         msg(1:len) = 'wrong PNM format'
         return
      endif

      read(IOUNIT, *) size_x_d, size_y_d
      if (size_x_d .gt. MAX_WIDTH) then
         call too_large(8, 'size_x_d', size_x_d, ier, len, msg)
         return
      endif
      width = size_x_d
      height = size_y_d
      area = width * height

      write(OUTPUT, 1001) width, height, area

 1001 format('width = ', i2, ', height = ', i2, ', area = ', i4)

      size_d = width * height
      if (size_d .gt. MAX_PIXEL_NUMBER) then
         call too_large(6, 'size_d', size_d, ier, len, msg)
         return
      endif

      read(IOUNIT, *) maxval

      i = 0
      do iy = 1, height
         read(IOUNIT, *) (line(k), k = 1, dim_d*width)
         k = 0
         do ix = 1, width
            i = i + 1
C           i - 1 = (iy - 1)*width + (ix - 1)
            do j = 1, dim_d
               k = k + 1
C              k - 1 = (ix - 1)*dim_d + (j - 1)
               d(i, j) = line(k) * 255d0 / maxval
               if (get_use_mask() .and. (mod(ix + iy, 2) .eq. 1)) then
                  keep(i, j) = .False.
               else
                  keep(i, j) = .True.
               endif
            enddo
         enddo
      enddo

      do j = 1, dim_d
         k = 0
         do i = 1, size_d
            if (keep(i, j)) then
               k = k + 1
               d(k, j) = d(i, j)
            endif
         enddo
      enddo
      size_d = k

      selected_size = size_d

      write(OUTPUT, 1002) selected_size

 1002 format('selected_size = ', OUTPUT_FLOAT_FMT)

      do j = 1, dim_d
         do i = 1, size_d
            write(OUTPUT, 1003) i, j, d(i, j)
         enddo
      enddo

 1003 format('i = ', i2, ', j = ', i2, ', d = ', OUTPUT_FLOAT_FMT)

      return
      end


      subroutine identity_checkerboard_masked_partition(mp)
C     Checkerboard optional mask.

      implicit none

      include 'model_definition.fh'

C     Output arguments.
      integer mp(MAX_PIXEL_NUMBER, MAX_CHANNEL_NUMBER)

C     Local variables.
      integer i, j

      do j = 1, dim_d
         do i = 1, size_x_d * size_y_d
            if (keep(i, j)) then
               mp(i, j) = 0
            else
               mp(i, j) = -1
            endif
         enddo
      enddo

      do j = 1, dim_d
         do i = 1, size_x_d * size_y_d
            write(OUTPUT, 1001) i, j, mp(i, j)
         enddo
      enddo

 1001 format('i = ', i2, ', j = ', i2, ', mp = ', OUTPUT_FLOAT_FMT)

      return
      end


      subroutine gradient
     +     (channel_number, pixel_number, fine_p, fine_g, ier, len, msg)
C     Gradient.

      implicit none

      include 'model_definition.fh'

C     Input arguments.
      integer channel_number, pixel_number
      real*8 fine_p(MAX_PIXEL_NUMBER, MAX_CHANNEL_NUMBER)

C     Output arguments.
      character msg*(*)
      integer ier, len
      real*8 fine_g(MAX_PIXEL_NUMBER, MAX_CHANNEL_NUMBER)

C     Local variable.
      integer i, j

      if (channel_number .ne. dim_d) then
         ier = 1
         len = 30
         msg(1:len) = 'wrong fine parameter dimension'
         return
      elseif (pixel_number .ne. size_d) then
         ier = 1
         len = 25
         msg(1:len) = 'wrong fine parameter size'
         return
      endif

      do j = 1, dim_d
         do i = 1, size_d
            fine_g(i, j) = fine_p(i, j) - d(i, j)
         enddo
      enddo

      return
      end


      subroutine optim(channel_number, pixel_number, zone_number,
     +     partition, cp_len, coarse_p, cost, ier, len, msg)
C     Optim.

      implicit none

      include 'model_definition.fh'

C     Input arguments.
      integer channel_number, pixel_number, zone_number
      integer partition(MAX_PIXEL_NUMBER, MAX_CHANNEL_NUMBER)
      integer cp_len(MAX_CHANNEL_NUMBER)

C     Input/output argument.
      real*8 coarse_p(MAX_ZONE_NUMBER)

C     Output arguments.
      character msg*(*)
      integer ier, len
      real*8 cost

C     Local variables.
      integer count(MAX_ZONE_NUMBER), i, j, k, k0, k1
      real*8 p_ij

      if (zone_number .gt. MAX_ZONE_NUMBER) then
         call too_large(11, 'zone number', zone_number, ier, len, msg)
         return
      elseif (channel_number .ne. dim_d) then
         ier = 1
         len = 30
         msg(1:len) = 'wrong fine parameter dimension'
         return
      elseif (pixel_number .ne. size_d) then
         ier = 1
         len = 25
         msg(1:len) = 'wrong fine parameter size'
         return
      endif

      do k = 1, zone_number
         coarse_p(k) = 0d0
         count(k) = 0
      enddo

C     Partition partition gives zone numbers from 0
C     and components of coarse parameter go from 1.
C     Hence, k0 is initialized to 1, and not 0.
      k0 = 1
      do j = 1, dim_d
         do i = 1, size_d
            k = k0 + partition(i, j)
            coarse_p(k) = coarse_p(k) + d(i, j)
            count(k) = count(k) + 1
         enddo
         k0 = k0 + cp_len(j)
      enddo

      k = 1
      do j = 1, dim_d
         do k1 = 1, cp_len(j)
            write(OUTPUT, 1001) k1, j, coarse_p(k), count(k)
            k = k + 1
         enddo
      enddo

 1001 format('k1 = ', i2, ', j = ', i2,
     +     ', m = ', OUTPUT_FLOAT_FMT, ', count = ', i2)

      do k = 1, zone_number
         coarse_p(k) = coarse_p(k) / dble(count(k))
      enddo

      k0 = 1
      cost = 0d0
      do j = 1, dim_d
         do i = 1, size_d
            k = k0 + partition(i, j)
            p_ij = coarse_p(k)
            cost = cost + (p_ij - d(i, j)) ** 2
         enddo
         k0 = k0 + cp_len(j)
      enddo
      cost = 0.5d0 * cost

      return
      end


      subroutine final
     +     (channel_number, pixel_number, fine_p, ier, len, msg)
C     Finalize the model.

      implicit none

      include 'model_definition.fh'

C     Input arguments.
      integer channel_number, pixel_number
      real*8 fine_p(MAX_PIXEL_NUMBER, MAX_CHANNEL_NUMBER)

C     Output arguments.
      character msg*(*)
      integer ier, len

      if (channel_number .ne. dim_d) then
         ier = 1
         len = 30
         msg(1:len) = 'wrong fine parameter dimension'
         return
      elseif (pixel_number .ne. size_d) then
         ier = 1
         len = 25
         msg(1:len) = 'wrong fine parameter size'
         return
      endif

      call save_pnm_image(fine_p, ier, len, msg)
      if (ier .ne. 0) return

      return
      end


      subroutine save_pnm_image(fine_p, ier, len, msg)
C     Save PNM image.

      implicit none

      include 'model_definition.fh'

C     Input argument.
      real*8 fine_p(MAX_PIXEL_NUMBER, MAX_CHANNEL_NUMBER)

C     Output arguments.
      character msg*(*)
      integer ier, len

C     Local variables.
      character magic*2

      open(unit=IOUNIT, file=fname_out(1:fno_len),
     +     status='unknown', access='sequential', form='formatted',
     +     iostat=ier)
      if (ier .ne. 0) then
         len = 22 + fno_len + 1
         msg(1:len) =
     +        'open failure on file "'//fname_out(1:fno_len)//'"'
         return
      endif

      if (dim_d .eq. 1) then
         magic = 'P2'
      else if (dim_d .eq. 3) then
         magic = 'P3'
      else
         ier = 1
         len = 24 + 1
         write(msg, 1001) dim_d
 1001    format('unbound channel number: ', i1)
         return
      endif
      write(IOUNIT, '(a2)') magic

      call save_ascii_pnm(fine_p)
      if (ier .ne. 0) return

      close(IOUNIT)

      return
      end


      subroutine save_ascii_pnm(fine_p)
C     Save ASCII PGM or PPM image.

      implicit none

      include 'model_definition.fh'

C     Input argument.
      real*8 fine_p(MAX_PIXEL_NUMBER, MAX_CHANNEL_NUMBER)

C     Local variables.
      integer maxval, ix, iy, iy1, i, j, k, line(3 * MAX_WIDTH)

      write(IOUNIT, '(a)')
     +     '# Generated by Ref-indic Fortran identity model.'

      write(IOUNIT, *) size_x_d, size_y_d

      maxval = 255
      write(IOUNIT, *) maxval

      iy1 = 1
      do iy = 1, size_y_d
         i = iy1
         k = 1
         do ix = 1, size_x_d
            do j = 1, dim_d
               line(k) = int(fine_p(i, j))
            enddo
            i = i + 1
            k = k + 1
         enddo
         write(IOUNIT, *) (line(k), k = 1, dim_d*size_x_d)
         iy1 = iy1 + size_x_d
      enddo

      return
      end


      subroutine too_large(n_len, name, bad, ier, len, msg)

      implicit none

C     Input arguments.
      character name*(*)
      integer n_len, bad

C     Output arguments.
      character msg*(*)
      integer ier, len

      ier = 1
      len = 10 + n_len + 2 + 1
      write(msg, 1001) name(1:n_len), bad

 1001 format('too large ', a, ': ', i1)

      return
      end


      subroutine set_use_mask

      implicit none

      include 'model_definition.fh'

      use_mask = .True.

      return
      end


      subroutine unset_use_mask

      implicit none

      include 'model_definition.fh'

      use_mask = .False.

      return
      end


      logical function get_use_mask()

      implicit none

      include 'model_definition.fh'

      get_use_mask = use_mask

      return
      end
